# -*- coding: utf-8 -*- 
#  -------------------------------------------------------------------
#  --- TILEsample_20201217.py ---
#  --- revision for NPi calc. (2020/12/17)
#  -------------------------------------------------------------------
import datetime
import numpy as np
import h5py
import cv2
import math

#   ------------------------------------------------------------------
Proc1st=datetime.datetime.now()
print("\n\n", Proc1st, " ... start of TILEsample")

DEG2RAD=(math.pi / 180.0)

#  -- parameter ---
foldername='X:\\'
filename='GC1SG1_20201214D01D_T0529_L2SG_LST_Q_2000.h5'
vv=np.int32(filename[21:23])
hh=np.int32(filename[23:25])

imggrp='Image_data'
sdname='LST'
img_png=sdname+'.png'
map_png=sdname+'_MAP.png'
map_tif=sdname+'_MAP.tif'
min_img=263     # -10 degC
max_img=303     # +30 degC
pngratio=0.001  # png ratio

print('target hdf=', filename)
print('vv,hh=', vv, hh)
print('target group/sdname=', imggrp, sdname)

geotif_lat = (34.0, 36.0)
geotif_lon = (139.0, 141.0)

print('geotif lat range=', geotif_lat)
print('geotif lon range=', geotif_lon)

#  open hdf and read sd
h5_file=h5py.File(foldername+filename, 'r')
sd=h5_file[imggrp][sdname]
(lines, pixels)=sd.shape

#  Attribute
Offset=sd.attrs['Offset']
Slope=sd.attrs['Slope']
MaxDN=sd.attrs['Maximum_valid_DN']
MinDN=sd.attrs['Minimum_valid_DN']

print('pixels={:}, lines={:}'.format(pixels, lines))
print('Slope={:}/Offset={:}'.format(Slope,Offset))
print('MinDN={:}/MaxDN={:}'.format(MinDN,MaxDN))

#  Engineering value conversion
img=sd[:].astype('float32')
img[np.where(img>MaxDN)]=np.nan
if MinDN > 0: img[np.where(img<MinDN)]=np.nan
img=img*Slope+Offset
del sd
print('min/max=',np.nanmin(img), np.nanmax(img))

#  save original PNG
dsp=img.reshape(lines,pixels,1)
dsp=(dsp-min_img)/(max_img-min_img)*255.0
dsp=cv2.applyColorMap(dsp.astype('uint8'),cv2.COLORMAP_JET)
dsize=(int(lines*pngratio), int(pixels*pngratio))
cv2.resize(dsp,dsize)
cv2.imwrite(img_png,dsp)
del dsp

#  tile projection constants
tileCols = 4800     # tile width for Q
tileRows = 4800     # tile height for Q
degStep = 180.0 / (18.0 * tileRows)      # grid step
NP0 = np.int32(2.0 * (180.0 / degStep))  # equator size

print("grid step [deg/col or deg/row] : ", degStep)
print("equator size [col] : ", NP0)

# global tile col, row
tileCol1st = hh * tileCols;
tileColEnd = tileCol1st + tileCols - 1;

tileRow1st = vv * tileRows;
tileRowEnd = tileRow1st + tileRows - 1;

#  calc lat lon for corners (UL, UR, LL, LR)
tileLatMax = 90.0 - degStep * np.int32(tileRow1st + 0.5)
tileLatMin = 90.0 - degStep * np.int32(tileRowEnd + 0.5)

NPi_max = np.float32(NP0) * np.cos(tileLatMax * DEG2RAD)  # rev 2020/12/17
NPi_min = np.float32(NP0) * np.cos(tileLatMin * DEG2RAD)  # rev 2020/12/17

tileLonUL = 360.0 / NPi_max * np.int32((tileCol1st - NP0 / 2) + 0.5)
tileLonUR = 360.0 / NPi_max * np.int32((tileColEnd - NP0 / 2) + 0.5)
tileLonLL = 360.0 / NPi_min * np.int32((tileCol1st - NP0 / 2) + 0.5)
tileLonLR = 360.0 / NPi_min * np.int32((tileColEnd - NP0 / 2) + 0.5)

tileLonUL = degStep * np.int32(tileLonUL / degStep + 0.5) # common grid
tileLonUR = degStep * np.int32(tileLonUR / degStep + 0.5)
tileLonLL = degStep * np.int32(tileLonLL / degStep + 0.5)
tileLonLR = degStep * np.int32(tileLonLR / degStep + 0.5)

tileLonMin = min(min(tileLonUL, tileLonLL), min(tileLonLL, tileLonLR))
tileLonMax = max(max(tileLonUL, tileLonUR), max(tileLonLL, tileLonLR))

mapCols = np.int32((tileLonMax - tileLonMin) / degStep + 0.5) + 1
mapRows = np.int32((tileLatMax - tileLatMin) / degStep + 0.5) + 1

print("tile lat range [deg] = ", (tileLatMin, tileLatMax))
print("tile lon range [deg] = ", (tileLonMin, tileLonMax))
print("dest map size (cols, rows) = ", (mapCols, mapRows))

#  destination array
dst=np.empty((mapRows,mapCols),dtype=np.float32)*np.nan

#  projection loop
print("... start of projection loop", end='')
for mapRow in range(mapRows):
    if np.mod(mapRow, 10)==0:
        if np.mod(mapRow, 500)==0: 
            print('\nmapRow[{:04d}/{:04d}] '.format(mapRow+1,mapRows), end='')
        print('.', end='')

    #  max row -> max lat
    lat = tileLatMax - mapRow * degStep;
    NPi = np.float32(NP0) * np.cos(lat * DEG2RAD)    # rev 2020/12/17
    rowEQA = np.int32(((90.0 - lat) / degStep - 0.5)) - tileRow1st;
    if (rowEQA < 0 or rowEQA >= mapRows): continue

    for mapCol in range(mapCols):
        #  min col -> min lon
        lon = tileLonMin + mapCol * degStep
        colEQA = np.int32(lon * NPi / 360.0 + NP0 / 2.0 - 0.5) - tileCol1st
        if (colEQA < 0 or colEQA >= tileCols): continue

        #  Pixel value
        dst[mapRow, mapCol] = img[rowEQA, colEQA]

del img
print("\n... end of projection loop")

#  save map PNG
dsp=dst.reshape(mapRows,mapCols,1)
dsp=(dsp-min_img)/(max_img-min_img)*255.0
dsp=cv2.applyColorMap(dsp.astype('uint8'),cv2.COLORMAP_JET)
dsize=(int(mapRows*pngratio), int(mapCols*pngratio))
cv2.resize(dsp,dsize)
cv2.imwrite(map_png,dsp)
del dsp
print("\n... end of map png output")

#  create GeoTiff
import gdal, osr      # for GeoTiff

dst_col = (geotif_lon - tileLonMin) / degStep
dst_row = (geotif_lat - tileLatMin) / degStep
dst_col = (np.int(dst_col[0]), np.int(dst_col[1]))
dst_row = (np.int(dst_row[0]), np.int(dst_row[1]))
print("geotiff col, row =", (dst_col, dst_row))

dst = dst[dst_row[0]:dst_row[1],dst_col[0]:dst_col[1]]
dst_shape = dst.shape
nXSize=np.int(dst_shape[1])
nYSize=np.int(dst_shape[0])

driver=gdal.GetDriverByName('GTiff')
outRaster=driver.Create(map_tif, nXSize, nYSize, 1, gdal.GDT_Float32)
outRaster.SetGeoTransform((geotif_lon[0], degStep, 0, geotif_lat[1], 0, -degStep))
outband = outRaster.GetRasterBand(1)
outband.SetNoDataValue(np.nan)
outband.WriteArray(np.array(dst))
outRasterSRS=osr.SpatialReference()
outRasterSRS.ImportFromEPSG(4326)
outRaster.SetProjection(outRasterSRS.ExportToWkt())
outband.FlushCache()
outband=None
del driver
del outband
print("\n... end of geotiff output")

print("dst.dtype=", dst.dtype)
print("dst.shape=", dst.shape)
print("min/max=", np.nanmin(dst), np.nanmax(dst))
print("nXSize,nYSize=",nXSize,nYSize)

# --- end of process ---
del dst
print()

ProcEnd=datetime.datetime.now()
print(ProcEnd, "... end of TILEsample ellapse time=", ProcEnd-Proc1st)
print("\n\n")
